* -------------------------------
* 	LPS Example
*
* 	Routine 5
*
*	Language: MC68K-ASM1
*	Style: plain MC68000
* 	Version: LPS20-L21-m68k
* -------------------------------

* Lecture L21 Plain MC68000 Solution

*  Questo esempio mostra la traduzione in ASM di un programma C
;	contenente una funzione ricorsiva

* Questo esempio e' realizzato per l'assembler ASM-One V1.48

* Variabili
; il tipo int viene memorizzato con word
	org	$4000

; la word seguente contiene il valore che la versione Plain C
; legge da standard input
t:	dc.w	12

; la word seguente, al termine del programma contiene il
; valore che la versione Plain C stampa su standard output
r:	dc.w	0

* Programma
	org	$2000

inizio_codice:

* Routine main
main:
	* input
	; la lettura mediante scanf della versione Plain C
	; viene simulata dalla lettura della word a indirizzo t
	
	* la routine main, in fase di calcolo
	; usa le variabili t ed r in memoria
	move.w	#3,d0	; chiama f( 3 )
	bsr		f		
	move.w	d1,d3	; valore di f( 3 ) in d3
	
	move.w	t,d0	; chiama f( t )
	bsr		f
	add.w	d1,d3	; f( 3 ) + f( t ) in d3
	
	move.w	d3,r	; copio risultato in r
	
	* output
	; la scrittura mediante printf della versione Plain C
	; viene simulata dalla scrittura della word a indirizzo r

fine_main:

	* fine programma
	bra	fine_codice

* Routine f
; Input
;	d0 (word)	- parametro
; Output
;	d1 (word)	- risultato
; Registri modificati
; Stack frame
; offset da indirizzo	| contenuto				| note
; area stack frame		|						|
;						|						|
;		0				| d0					| inserito da f
;						|						|
; 		2				| indirizzo di ritorno	| inserito da bsr
;						|						|

f:
	cmp.w	#1,d0
	blt		f_caso_base
	
	; alloca spazio per d0 e salva d0
	move.w	d0,-(sp)
	
	; chiamata ricorsiva
	asr.w	#1,d0
	bsr	f
	; d1 contiene il risultato della chiamata ricorsiva
	
	; ripristino d0 e dealloco lo spazio dallo stack
	move.w	(sp)+,d0
	sub.w	d0,d1
	
	rts
	
	
f_caso_base:
	move.w	#1,d1
	rts

fine_codice:

	end

* Original Plain C Code

#include <stdio.h>

int f( int x ) {
	if ( x < 1 ) return 1;
	else return f( x / 2 ) - x;
}

int main( void ) {

	int r, t;

	/* input */
	scanf( "%d", &t );

	r = f( t ) + f( 3 );

	/* output */
	printf( "%d\n", r );

	return 0;
}

/*
	Test Cases

	Test 1
	input: 0
	output: 1

	Test 2
	input: 1
	output: 0

	Test 3
	input: 2
	output: -2

	Test 4
	input: 3
	output: -3

	Test 5
	input: 8
	output: -14

	Test 6
	input: 12
	output: -21

	Test 7
	input: 16
	output: -30

	Test 8
	input: 20
	output: -37
*/
