# -------------------------------
# 	LPS Example
#
# 	Routine 5
#
#	Language: MIPS32-MARS
#	Style: plain MIPS32
# 	Version: LPS20-L21-mips
# -------------------------------

#  Lecture L21 Plain MIPS32 Solution

#  Questo esempio mostra la traduzione in ASM di un programma C
#	contenente una funzione ricorsiva

# Questo esempio e' realizzato per l'assembler MARS
# Per eseguire il programma si possono usare i comandi della GUI

# Variabili
# il tipo int viene memorizzato con word
	.data

# la word seguente contiene il valore che la versione Plain C
# legge da standard input
t:	.word	0

# la word seguente, al termine del programma contiene il
# valore che la versione Plain C stampa su standard output
r:	.word	0

# Programma
	.text

inizio_codice:

# Routine main
main:
	# input
	# la lettura mediante scanf della versione Plain C
	# viene simulata dalla lettura della word a indirizzo t
	
	# uso dei registri (routine main, fase di calcolo)
	#	- variabile t
	#	- variabile r
	# chiamata di f( 3 )
	li	$a0,3
	jal	f
	move	$s0,$v0	# copio f( 3 ) in s0

	# chiamata di f( t )
	lw	$a0,t
	jal	f
	add	$s0,$s0,$v0	# pongo in s0 f( 3 ) + f( t )
	sw	$s0,r
				
	# output
	# la scrittura mediante printf della versione Plain C
	# viene simulata dalla scrittura della word a indirizzo r

fine_main:

	# fine programma
	j	fine_codice

# Routine f
# Input
#	a0 - parametro
# Output
#	v0 - risultato
# Registri modificati
#	v0
# Stack frame
# offset da indirizzo	| contenuto	| note
# area stack frame	|		|
#			|		|
# 	0		| ra		|
#			|		|
#	4		| a0		|
#			|		|
# dimensione stack frame: 8
f:
	blt	$a0,1,f_caso_base

	# alloco nello stack spazio per uno stack frame
	sub	$sp,$sp,8
	# salvo nello stack ra e a0
	sw	$ra,($sp)
	sw	$a0,4($sp)

	# poichè a0 ha un valore positivo
	# lo shift a destra produce esattamente lo stesso
	# risultato di una divisione mediante div
	sra	$a0,$a0,1
	jal	f
	# il risultato di f è in v0
	# devo sottrarre a v0 il valore di a0 prima della jal

	# ripristino il valore di a0 precedente la jal
	lw	$a0,4($sp)
	
	# calcolo il risultato
	sub	$v0,$v0,$a0

	# ripristino ra
	lw	$ra,($sp)
	add	$sp,$sp,8

	jr	$ra

f_caso_base:
	li	$v0,1
	jr	$ra

fine_codice:

# Original Plain C Code
#
# #include <stdio.h>
#
# int f( int x ) {
#	if ( x < 1 ) return 1;
#	else return f( x / 2 ) - x;
# }
#
# int main( void ) {
#
#	int r, t;
#
#	/* input */
#	scanf( "%d", &t );
#
#	r = f( t ) + f( 3 );
#
#	/* output */
#	printf( "%d\n", r );
#
#	return 0;
# }
#
# /*
#	Test Cases
#
#	Test 1
#	input: 0
#	output: -2
#
#	Test 2
#	input: 1
#	output: -3
#
#	Test 3
#	input: 2
#	output: -5
#
#	Test 4
#	input: 3
#	output: -6
#
#	Test 5
#	input: 8
#	output: -17
#
#	Test 6
#	input: 12
#	output: -24
#
#	Test 7
#	input: 16
#	output: -33
#
#	Test 8
#	input: 20
#	output: -40
# */
